package ROTool::AcoNavi::Client;
use strict;
use ROTool;
use vars qw(@ISA);

my $APPRI_VER = "AcoNavi";
@ISA = qw( ROTool::Client ROTool );

my @field_order;
my %field_aco2rotool;
my %field_rotool2aco;

sub typechecker{
    #not a method.
    my $self = shift;
    if($self->{"buf"} =~ /set\001version\0011\001\002/){
	bless $self;
	$self->new();
	return 1;
    }else{
	return 0;
    }
}

sub import{
    &ROTool::typechecker_regist(\&typechecker);

    @field_order = ("id","gameservername","areaname","name", "hp","hpmax",
		    "sp","spmax","baselv","joblv","classname","exp","expmax",
		    "jobexp","jobexpmax","weight","weightmax","expperhour",
		    "time","text");
    %field_aco2rotool
	= ("id"             => "id",
	   "name"           => "name",
	   "areaname"       => "area",
	   "gameservername" => "server",
	   "hp"             => "hp",
	   "hpmax"          => "hpmax",
	   "sp"             => "sp",
	   "spmax"          => "spmax",
	   "baselv"         => "baselv",
	   "joblv"          => "joblv",
	   "exp"            => "exp",
	   "expmax"         => "expmax",
	   "jobexp"         => "jobexp",
	   "jobexpmax"      => "jobexpmax",
	   "expperhour"     => "expperhour",
	   "classname"      => "class",
	   "weight"         => "weight",
	   "weightmax"      => "weightmax",
	   "time"           => "time",
	   "text"           => "message",
	   "version"        => "aconavi_version");
    foreach(keys %field_aco2rotool){
	my($aco,$rotool) = ($_,$field_aco2rotool{$_});
	$field_rotool2aco{$rotool} = $aco;
    }
}

sub new{
    my $class = shift;
    my $self = $class->SUPER::new(@_);
    $self->{"param"}->{"APPRI_VER"} = $APPRI_VER;
    foreach(values %field_aco2rotool){
	$self->{"param"}->{$_} = "";
    }
    return $self;
}
    
sub is_client{
    return 1;
}

sub is_server{
    return 0;
}

sub recv{
    my($self) = shift;

    my($ret) = $self->SUPER::recv();
    &ROTool::printlog(2," [$self->{oid}] parse message\n");
    while($self->{"buf"} =~ s/^(.*?)\002//){
	my $msg = $1;
	my($set,$field,$value) = split(/\001/,$msg);
	if(exists $field_aco2rotool{$field}){
	    $self->{"param"}->{$field_aco2rotool{$field}} = $value;
	}
    }
    $self->{"param"}->{"time"} = sprintf("%02d:%02d", (localtime())[2,1]);
    return $ret if(not defined $self->{"param"}->{"id"});
    if(&ROTool::UA_regist($self->{"param"}->{"id"},$self) == 0){
	return 0;
    }
    return $ret;
}

sub send{
    my $self = shift;
    my @msg;
    foreach my $client(&ROTool::UA_list()){
	my(@line) = ("Unknown");
	foreach my $field (@field_order){
	    my $value;
	    $value = $client->{"param"}->{$field_aco2rotool{$field}} || "";
	    push(@line,$value);
	}
	push(@msg,join("\003", @line));
    }
    push(@msg,"\002");
    my $msg = join("\004",@msg);
    my $sock = $self->{"sock"};
    $sock->syswrite($msg,length($msg));
    &ROTool::printlog(2,$msg."\n");
}

sub is_keep_message{
    my $self = shift;
    return 0 if($self->{"Message_lasting"});
    return 1 if($self->{"param"}->{"message"}
		=~ /^(\(|\x81\x69).*?(\)|\x81\x6a)$/);
    return 0;
}

1;
